"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InstanceFactory = void 0;
const errorfactory_1 = require("../tools/errorfactory");
const util_1 = require("../tools/util");
const application_1 = require("../tools/application");
const aopfactory_1 = require("./aopfactory");
/**
 * 实例工厂
 * @remarks
 * 用于管理所有的实例对象
 */
class InstanceFactory {
    /**
     * 工厂初始化
     * @param config    配置项
     */
    static async init(config) {
        await this.parse(config);
        aopfactory_1.AopFactory.proxyAll();
    }
    /**
     * 添加单例到工厂
     * @param cfg       实例配置对象
     */
    static addInstance(clazz, cfg) {
        if (!this.factory.has(clazz)) { //新建
            this.factory.set(clazz, { singleton: true });
        }
        //参数赋值
        if (cfg) {
            let cfg1 = this.factory.get(clazz);
            cfg1.singleton = cfg.singleton !== false;
            cfg1.params = cfg.params;
            cfg1.instance = cfg.instance;
        }
    }
    /**
     * 注入
     * @param targetClass       目标类
     * @param propName          目标属性名
     * @param injectClass       注入类
     */
    static inject(targetClass, propName, injectClass) {
        if (!this.factory.has(targetClass)) {
            this.factory.set(targetClass, {
                properties: new Map()
            });
        }
        let cfg = this.factory.get(targetClass);
        if (!cfg.properties) {
            cfg.properties = new Map();
        }
        cfg.properties.set(propName, injectClass);
    }
    /**
     * 获取实例
     * @param name  实例名
     * @param param 参数数组
     * @returns     实例化的对象或null
     */
    static getInstance(clazz, param) {
        let ins = this.factory.get(clazz);
        if (!ins) {
            return null;
        }
        if (ins.singleton && ins.instance) {
            return ins.instance;
        }
        else {
            param = param || ins.params || [];
            let instance = Reflect.construct(clazz, param);
            if (ins.singleton) {
                ins.instance = instance;
            }
            if (ins.properties) {
                for (let k of ins.properties) {
                    instance[k[0]] = this.getInstance(k[1]);
                }
            }
            return instance;
        }
    }
    /**
     * 获取实例配置对象
     * @param clazz  实例名
     * @returns             实例配置对象
     */
    static getInstanceCfg(clazz) {
        return this.factory.get(clazz);
    }
    /**
     * 执行实例的一个方法
     * @param instance     实例类或实例对象
     * @param methodName    方法名
     * @param params        参数数组
     * @param func          方法(与methodName二选一)
     * @returns             方法对应的结果
     */
    static exec(instance, methodName, params, func) {
        //instance是类
        let instanceName = '';
        if (instance && typeof (instance) === 'function') {
            instanceName = instance.name;
            instance = this.getInstance(instance);
        }
        //实例不存在
        if (!instance) {
            throw new errorfactory_1.NoomiError("1001", instanceName);
        }
        func = func || instance[methodName];
        //方法不存在
        if (!func) {
            throw new errorfactory_1.NoomiError("1010", methodName);
        }
        return func.apply(instance, params);
    }
    /**
     * @exclude
     * 处理配置对象
     * @param
     */
    static parse(path) {
        if (Array.isArray(path)) {
            for (let p of path) {
                handle(p);
            }
        }
        else {
            handle(path);
        }
        /**
         * 处理instance路径
         * @param path  待解析路径
         */
        function handle(path) {
            const basePath = process.cwd();
            let pathArr = path.split('/');
            let pa = [basePath];
            let handled = false; //是否已处理
            for (let i = 0; i < pathArr.length - 1; i++) {
                const p = pathArr[i];
                if (p.indexOf('*') === -1 && p !== "") {
                    pa.push(p);
                }
                else if (p === '**') { //所有子孙目录
                    handled = true;
                    if (i < pathArr.length - 2) {
                        throw new errorfactory_1.NoomiError('1000');
                    }
                    handleDir(pa.join('/'), pathArr[pathArr.length - 1], true);
                }
            }
            if (!handled) {
                handleDir(pa.join('/'), pathArr[pathArr.length - 1]);
            }
            /**
             * 处理子目录
             * @param dirPath   目录地址
             * @param fileExt   文件后缀
             * @param deep      是否深度处理
             */
            function handleDir(dirPath, fileExt, deep) {
                const dir = application_1.App.fs.readdirSync(dirPath, { withFileTypes: true });
                let fn = fileExt;
                let reg = util_1.Util.toReg(fn, 3);
                for (const dirent of dir) {
                    if (dirent.isDirectory()) {
                        if (deep) {
                            handleDir(application_1.App.path.resolve(dirPath, dirent.name), fileExt, deep);
                        }
                    }
                    else if (dirent.isFile()) {
                        // @Instance注解方式文件，自动执行instance创建操作
                        if (reg.test(dirent.name)) {
                            // require(App.path.resolve(dirPath , dirent.name));
                            Promise.resolve().then(() => require(application_1.App.path.resolve(dirPath, dirent.name)));
                        }
                    }
                }
            }
        }
    }
    /**
     * 类是否已添加到工厂
     * @param clazz     类对象
     * @returns         true/false
     */
    static hasClass(clazz) {
        return this.factory.has(clazz);
    }
    /**
     * 获取instance工厂
     * @returns     实例工厂
     */
    static getFactory() {
        return this.factory;
    }
}
exports.InstanceFactory = InstanceFactory;
/**
 * 实例工厂map，存放所有实例对象
 * key：类
 */
InstanceFactory.factory = new Map();
//# sourceMappingURL=instancefactory.js.map