/**
 * Aop通知类型
 */
interface IAopAdvice {
    /**
     * 切点
     */
    pointcutId?: string;
    /**
     * 类名
     */
    clazz?: any;
    /**
     * 通知类型 (before,after,after-return,after-throw,around)
     */
    type: string;
    /**
     * 对应的切面方法
     */
    method: string;
}
/**
 * Aop切面类型
 */
interface IAopAspect {
    /**
     * 实例名
     */
    instance?: string;
    /**
     * 切点数组
     */
    pointcuts: Array<AopPointcut>;
    /**
     * 通知数组
     */
    advices: Array<IAopAdvice>;
}
/**
 * 切点数据对象
 */
interface IAopPointcut {
    /**
     * 类
     */
    clazz: any;
    /**
     * 切点id
     */
    id: string;
    /**
     * 表达式串
     */
    expressions?: Array<string>;
}
/**
 * @exclude
 * aop文件配置对象
 */
interface IAopCfg {
    files: Array<string>;
    pointcuts: Array<IAopPointcut>;
    aspects: Array<IAopAspect>;
}
/**
 * aop 切点类
 */
declare class AopPointcut {
    /**
     * 切点id
     */
    id: string;
    /**
     * 实例名
     */
    /**
     * 切面类
     */
    aspectClazz: any;
    /**
     * 表达式数组（正则表达式）
     */
    expressions: Array<RegExp>;
    /**
     * 通知数组
     */
    advices: Array<IAopAdvice>;
    /**
     * 构造器
     * @param id            切点id(实例内唯一)
     * @param instanceName  实例名
     * @param expressions   该切点对应的表达式数组，表达式为正则表达式串
     */
    constructor(id: string, expressions: Array<string>, clazz: any);
    /**
     * 给切点添加通知
     * @param advice    通知对象
     */
    addAdvice(advice: IAopAdvice): void;
    /**
     * 添加表达式串
     * @param expr  表达式串或数组
     */
    addExpression(expr: string | string[]): void;
}
/**
 * Aop工厂
 * 用于管理所有切面、切点
 */
declare class AopFactory {
    /**
     * 切点map，用于存储所有切点
     * key:切面类名.切点名
     */
    private static pointcuts;
    /**
     * 已代理方法map，键为instanctName.methodName，避免重复代理
     * @since 0.4.4
     */
    private static proxyMethodMap;
    /**
     * 注册切面map，键为className,值为
     *          {
     *              isAspect:true,  //避免用了pointcut，但是未使用Aspect注解
     *              pointCutId1:{expressions:Array<string>,advices:{type:类型,method:方法名}},
     *              ...
     *          }
     */
    private static registAspectMap;
    /**
     * 注册切点
     * @param cfg   pointcut配置
     * @since 1.0.0
     */
    static registPointcut(cfg: IAopPointcut): void;
    /**
     * 注册advice
     * @param cfg   advice配置
     * @since 1.0.0
     */
    static registAdvice(cfg: IAopAdvice): void;
    /**
     * 从registAspectMap中获取注册的pointcut配置
     * @param className     类名
     * @param pointcutId    切点id
     * @param create        如果不存在，是否创建，如果为true，则创建，默认false
     * @returns             pointcut配置项
     * @since 1.0.0
     */
    private static getRegistPointcut;
    /**
     * 处理切面
     * @param clazz     切面类
     * @since 1.0.0
     */
    static addAspect(clazz: any): void;
    /**
     * 为切点添加表达式
     * @param pointcutId    切点id
     * @param expression    表达式或数组
     */
    static addExpression(pointcutId: string, expression: string | Array<string>): void;
    /**
     * 为所有aop匹配的方法设置代理
     */
    static proxyAll(): void;
    /**
     * 为某个类设置代理
     * @param instanceName  实例名
     * @param clazz         类
     */
    static proxyOne(clazz: any): void;
    /**
     * 根据id获取切点
     * @param pointcutId    切点id
     * @returns             切点对象
     */
    static getPointcutById(pointcutId: string): AopPointcut;
    /**
     * 获取advices
     * @param clazz         类
     * @param methodName    方法名
     * @return              {
     *                          before:[{instance:切面实例,method:切面方法},...]
     *                          after:[{instance:切面实例,method:切面方法},...]
     *                          return:[{instance:切面实例,method:切面方法},...]
     *                          throw:[{instance:切面实例,method:切面方法},...]
     *                      }
     */
    static getAdvices(clazz: any, methodName: string): object;
}
export { AopFactory, IAopAdvice, IAopAspect, AopPointcut, IAopCfg, IAopPointcut };
