/**
 * 实例属性
 */
export interface IInstanceProperty {
    /**
     * 属性名
     */
    name: string;
    /**
     * 引用实例名
     */
    ref: string;
}
/**
 * 实例配置对象
 */
interface IInstanceCfg {
    /**
     * 实例与path 二选一
     */
    instance?: any;
    /**
     * 单例模式，如果为true，表示该类只创建一个实例，调用时，共享调用
     */
    singleton?: boolean;
    /**
     * 参数数组，初始化时需要传入的参数
     */
    params?: Array<any>;
    /**
     * 属性列表，定义需要注入的属性
     */
    properties?: Array<IInstanceProperty>;
}
/**
 * 实例对象，实例工厂中的存储元素
 */
export interface IInstance {
    /**
     * 实例对象
     */
    instance?: any;
    /**
     * 类引用
     */
    class?: any;
    /**
     * 单例标志
     */
    singleton?: boolean;
    /**
     * 构造器参数
     */
    params?: Array<any>;
    /**
     * 需要注入的属性列表
     */
    properties?: Map<string, any>;
}
/**
 * 注入参数对象，用于存储待注入对象的参数
 */
export interface IInject {
    /**
     * 待注入实例
     */
    instance?: any;
    /**
     * 待注入类
     */
    clazz?: any;
    /**
     * 待注入属性名
     */
    propName: string;
    /**
     * 注入实例名
     */
    injectName: string;
}
/**
 * 实例工厂
 * @remarks
 * 用于管理所有的实例对象
 */
export declare class InstanceFactory {
    /**
     * 实例工厂map，存放所有实例对象
     * key：类
     */
    static factory: Map<any, IInstance>;
    /**
     * 工厂初始化
     * @param config    配置项
     */
    static init(config: any): Promise<void>;
    /**
     * 添加单例到工厂
     * @param cfg       实例配置对象
     */
    static addInstance(clazz: any, cfg?: IInstanceCfg): any;
    /**
     * 注入
     * @param targetClass       目标类
     * @param propName          目标属性名
     * @param injectClass       注入类
     */
    static inject(targetClass: any, propName: string, injectClass: any): void;
    /**
     * 获取实例
     * @param name  实例名
     * @param param 参数数组
     * @returns     实例化的对象或null
     */
    static getInstance(clazz: any, param?: Array<any>): any;
    /**
     * 获取实例配置对象
     * @param clazz  实例名
     * @returns             实例配置对象
     */
    static getInstanceCfg(clazz: any): IInstance;
    /**
     * 执行实例的一个方法
     * @param instance     实例类或实例对象
     * @param methodName    方法名
     * @param params        参数数组
     * @param func          方法(与methodName二选一)
     * @returns             方法对应的结果
     */
    static exec(instance: any, methodName: string, params: Array<any>, func?: Function): any;
    /**
     * @exclude
     * 处理配置对象
     * @param
     */
    private static parse;
    /**
     * 类是否已添加到工厂
     * @param clazz     类对象
     * @returns         true/false
     */
    static hasClass(clazz: any): boolean;
    /**
     * 获取instance工厂
     * @returns     实例工厂
     */
    static getFactory(): Map<any, IInstance>;
}
export {};
